/*******************************************************************************
 *
 * Thomas Thomassen
 * thomas[at]thomthom[dot]net
 *
 ******************************************************************************/


var FADE_DURATION = 300; // Milliseconds.

var SLOW_RESPONSE_THRESHOLD = 10000;
var RESPONSE_TIMEOUT = 60000;

var g_configuration = {};


function init_license_actions()
{
  $('#action-activate').on('click', function() {
    check_license();
  });

  $('#action-deactivate').on('click', function() {
    remove_license();
  });

  $('#action-license').on('click', function() {
    Sketchup.callback("LicenseErrorWindow.open_license_dialog");
  });

  // `propertychange` is fallback for IE8 users - enough number of them despite
  // SU2015 require minimum IE9.
  // There is also an IE9 bug where the event doesn't trigger when the textbox
  // becomes empty. Not a big deal though.
  $('#license-key').on('propertychange input', function() {
    var empty = $('#license-key').val().length === 0;
    $('#action-activate').prop('disabled', empty);
  });
}


$(document).ready(function() {
  init_license_actions();
});


function configure_window(config)
{
  g_configuration = config;
  display_license_info(g_configuration.license);
}


function remove_license()
{
  Sketchup.callback("LicenseWindow.license_remove");
}


function check_license()
{
  check_license_overlay();

  var host = g_configuration.server;
  var api = host + '/api/v1';

  var product_id = g_configuration.product_id;
  var key = $.trim($('#license-key').val());

  var request_uri = api + '/license/validate/' + product_id + '/' + key;

  //debugger;

  var timer = setTimeout(show_slow_reponse_warning, SLOW_RESPONSE_THRESHOLD);
  $.ajax({
    url: request_uri,
    dataType: "jsonp",
    timeout: RESPONSE_TIMEOUT,

    success: function(data) {
      Sketchup.callback("LicenseWindow.license_check", data);
    }
  })
  .fail(function(jqXHR, textStatus, errorThrown) {
    show_response_error("Failed to connect to license server.");
    var data = {
      textStatus : textStatus,
      errorThrown : errorThrown,
      timeoutThreshold: RESPONSE_TIMEOUT
    };
    Sketchup.callback('Window.log', data);
  })
  .always(function() {
    clearTimeout(timer);
    close_overlay();
  });
}


function show_response_error(message)
{
  var $error = $("<p class='error'>" + message + "</p>");
  $error.hide();
  $("#activate").append($error);
  $error.fadeIn(FADE_DURATION);
}


function show_slow_reponse_warning()
{
  var $warning = $("<p>Server slow to respond... :(</p>");
  $warning.hide();
  $("#overlay").append($warning);
  $warning.fadeIn(FADE_DURATION);
}


function check_license_overlay()
{
  // Remove any previous error messages.
  $("#activate .error").detach();
  // Disable the button.
  $("#action-activate").attr("disabled", true);
  // Create the loading animation.
  var $content = $("<div>");
  var $img = $("<img src='../images/working.gif'>");
  $content.append($img);
  // Display the overlay message.
  var $overlay = message_overlay($content);
  $overlay.fadeIn(FADE_DURATION);
}


function message_overlay($content)
{
  var $overlay = $("#overlay");
  if ($overlay.length == 0)
  {
    $overlay = $("<div id='overlay'/>");
    $("#content").append($overlay);
  }
  $overlay.hide();
  $overlay.empty();
  $overlay.append($content);
  return $overlay;
}


function close_overlay()
{
  $("#action-activate").attr("disabled", false);
  var $overlay = $("#overlay");
  $overlay.fadeOut(FADE_DURATION);
}


function display_license_info(license)
{
  $("h1 i").detach();

  if (license.valid && !license.trial)
  {
    $("#title").text("Licensed");
    $('#activate').hide();
    $('#deactivate').show();
  }
  else
  {
    $("#title").text("Not Licensed");
    $('#activate').show();
    $('#deactivate').hide();
  }

  if (license.trial)
  {
    var info = '';
    var now = Math.floor(Date.now() / 1000); // In seconds.
    if (license.expire < now)
    {
      info = 'Trial expired';
    }
    else
    {
      var seconds_left = license.expire - now;
      var days_left = Math.floor(seconds_left / 60 / 60 / 24);
      info = 'Trial expire in ' + days_left + ' days';
    }
    $("#title").after($('<i>' + info + '</i>'));
  }
}


function display_license_feedback(license_error_type, message)
{
  switch (license_error_type)
  {
  case "NotLicensedError":
    display_not_licensed_error();
    break;
  case "LicenseExpiredError":
    display_licensed_expired_error();
    break;
  case "TrialLicenseExpiredError":
    display_trial_licensed_expired_error();
    break;
  case "BetaBuildExpiredError":
    display_beta_expired_error();
    break;
  case "LicenseError":
    display_license_error(message);
    break;
  }
  //append_disclaimer();
}


function display_not_licensed_error()
{
  $("#title").text("Not Licensed");
}


function display_licensed_expired_error()
{
  $("#title").text("License Expired");
  var message = "\
  <p>\
    Your Vertex Tools license has expired. Please make sure you have a valid license.\
  </p>";
  $("#content").html(message);
}


function display_trial_licensed_expired_error()
{
  $("#title").text("Trial License Expired");
  var message = "\
  <p>\
    Your Vertex Tools trial license has expired. Please purchase a full license in\
    order to continue using the product.\
  </p>";
  $("#content").html(message);
}


function display_beta_expired_error()
{
  $("#title").text("Beta Build Expired");
  var message = "\
  <p>\
    Your Vertex Tools beta build has expired.\
  </p>";
  $("#content").html(message);
}


function display_license_error(error_message)
{
  var message = "\
  <p>\
    There was an unexpected error obtaining your license. Please make sure you\
    have a working internet connection and valid license.\
  </p>";
  $("#content").html(message);
  var $message = $("<p>" + error_message + "</p>");
  $("#content").append($message);
}


function append_disclaimer()
{
  var message = "\
  <p>\
    If you believe you see this message in error, then please contact\
    support.\
  </p>";
  var $message = $("<p>" + message + "</p>");
  $("#content").append($message);
}
